"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.run = void 0;

var _path = require("path");

var _fs = require("fs");

var _vinylFs = _interopRequireDefault(require("vinyl-fs"));

var chokidar = _interopRequireWildcard(require("chokidar"));

var _lodash = require("lodash");

var _gulpIf = _interopRequireDefault(require("gulp-if"));

var _gulpTypescript = _interopRequireDefault(require("gulp-typescript"));

var _through = _interopRequireDefault(require("through2"));

var babel = _interopRequireWildcard(require("@babel/core"));

var _utils = require("./utils");

var _getBabelConfig = require("./getBabelConfig");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const run = async (opts, ctx) => {
  const {
    cwd,
    watch,
    type = "cjs",
    srcDir = "src",
    output,
    target = "node",
    include = ["**/*"],
    exclude = ["**/__test__{,/**}", "**/*.+(test|e2e|spec).+(js|jsx|ts|tsx)"],
    runtimeHelpers,
    extraPresets = [],
    extraPlugins = [],
    nodeVersion,
    disableTypeCheck,
    lazy
  } = opts;
  const srcPath = (0, _path.resolve)(cwd, srcDir);
  const outputDir = output || type === "cjs" ? "lib" : "es";
  const outputPath = (0, _path.resolve)(cwd, outputDir);
  const patterns = include.map(p => (0, _path.join)(srcPath, p)).concat(exclude.map(p => `!${(0, _path.join)(srcPath, p)}`));

  const getTSConfig = () => {
    const tsConfigPath = (0, _path.join)(cwd, "tsconfig.json");
    const templateTSConfigPath = (0, _path.join)(__dirname, "../template/tsconfig.json");

    if ((0, _fs.existsSync)(tsConfigPath)) {
      return (0, _utils.getTSConfigCompilerOptions)(tsConfigPath) || {};
    }

    return (0, _utils.getTSConfigCompilerOptions)(templateTSConfigPath) || {};
  };

  const transform = file => {
    const babelOpts = (0, _getBabelConfig.getBabelConfig)({
      target,
      type,
      typescript: /\.tsx?$/.test(file.path),
      runtimeHelpers,
      nodeVersion,
      lazy
    });
    babelOpts.presets.push(...extraPresets);
    babelOpts.plugins.push(...extraPlugins);
    ctx.console.log(`Transform to ${type} for ${(0, _path.relative)(cwd, file.path)}`);
    return babel.transform(file.contents, { ...babelOpts,
      filename: file.path
    }).code;
  };

  const createStream = src => {
    const tsConfig = getTSConfig();
    const tsFileRegexp = /\.tsx?$/;
    const babelTransformRegexp = disableTypeCheck ? /\.(t|j)sx?$/ : /\.jsx?$/;
    return _vinylFs.default.src(src, {
      allowEmpty: true,
      base: srcPath
    }).pipe((0, _gulpIf.default)(f => !disableTypeCheck && (0, _utils.isTransform)(f.path, tsFileRegexp), (0, _gulpTypescript.default)(tsConfig))).pipe((0, _gulpIf.default)(f => (0, _utils.isTransform)(f.path, babelTransformRegexp), _through.default.obj((file, env, cb) => {
      try {
        file.contents = Buffer.from(transform(file)); // eslint-disable-line no-param-reassign

        file.path = file.path.replace((0, _path.extname)(file.path), ".js"); // eslint-disable-line no-param-reassign

        cb(null, file);
      } catch (e) {
        ctx.console.error(`Compiled failed: ${file.path}`);
        ctx.console.error(e);
        cb(null);
      }
    }))).pipe(_vinylFs.default.dest(outputPath));
  };

  return new Promise(resolvePromise => {
    createStream(patterns).on("end", () => {
      if (watch) {
        ctx.console.log(`Start watching ${(0, _path.relative)(cwd, srcPath)} directory...`);
        const watcher = chokidar.watch(patterns, {
          ignoreInitial: true
        });
        const files = [];
        const compileFiles = (0, _lodash.debounce)(() => {
          while (files.length) {
            createStream(files.pop());
          }
        }, 1000);
        watcher.on("all", (event, fullPath) => {
          ctx.console.log(`[${event}] ${(0, _path.relative)(cwd, fullPath)}`);

          if (!(0, _fs.existsSync)(fullPath)) {
            return;
          }

          if ((0, _fs.statSync)(fullPath).isFile()) {
            if (!files.includes(fullPath)) {
              files.push(fullPath);
            }

            compileFiles();
          }
        });
        process.once("SIGINT", () => {
          watcher.close();
        });
      }

      resolvePromise();
    });
  });
};

exports.run = run;